using AdventureClient.Help;
using AdventureLobby.RedisService;
using Config;
using Gate;
using Gate.WebClient;
using Help;
using LobbyServer;
using Log;
// using Manager;
using Module;
using ServerLib;
using ServerLib.StaticData;
using System;
using System.Diagnostics;
using System.Threading;
using Utility.Timer;

namespace AdventureClient
{
    //存放Server的全局引用的类
    public class Env
    {
        //服务器配置
        public static ServerConfig Config { get; private set; }
        //模块管理器
        public static IModuleManager ModuleManager { get; private set; }
        //定时任务
        public static JobScheduler JobTimer { get; private set; }
        //redis数据库
        public static RedisService redis { get; private set; }
        //mongo数据库
        public static MongoSystem mongodb = new MongoSystem();
        //读取配置
        public static OtherConfig otherConfig = new OtherConfig();
        //qq小游戏
        public static HttpQQMini qqMini = new HttpQQMini();
        //微信相关
        public static Weixin weixin = new Weixin();
        //客户端连接管理器
        // public static BackSessionManager backSessionMgr = new BackSessionManager();
        //网页客户端连接管理器
        public static WebClientSessionManager webClientSessionMgr = new WebClientSessionManager();

        public static SingleThreadSynchronizationContext m_SynchronizationContext;

        public static ITimerManager Timer { get; private set; }

        public static Event.IEventSystem eventSystem = new Event.EventSystem();

        //玩家管理器
        public static PlayerManager playerMgr = new PlayerManager();
        //httpPOST监听
        public static HttpListener httpResponse = new HttpListener();
        //http post upload上传相关
        public static HttpUploadListener httpUploadLis = new HttpUploadListener();

        //服务器开启时间
        public static DateTime ServerOpenTime = DateTime.MinValue;

        public static void Init(ServerConfig config)
        {
            Console.Title = Process.GetCurrentProcess().ProcessName;
            ThreadPoolHelper.OnInitThreadPool();
            Config = config;
            ModuleManager = new ModuleManager();
            JobTimer = new JobScheduler();
            redis = new RedisService();
            Timer = new TimerManager();
            mongodb = new MongoSystem();
            mongodb.Init();

            ModuleManager.RegisterModule(new WebClientModule());
            // ModuleManager.RegisterModule(new GameModule());
            // ModuleManager.RegisterModule(new GMModule());
            // ModuleManager.RegisterModule(new WorldModule());
            // ModuleManager.RegisterModule(new HomeModule());
            // ModuleManager.RegisterModule(new HouseSysModule());

        }

        public static bool IsAlive = true;

        public bool Start()
        {
            if (!Init())
            {
                Destroy();
                return false;
            }

            while (IsAlive)
            {
                //LogSys.Debug("Env.Start1");

                DateTime startTime = DateTime.Now;
                try
                {
                    Update();
                }
                catch (Exception ex)
                {
                    LogSys.ErrorModule("Exception", $"Exception: {ex.Message}. Stack {ex.StackTrace}");
                    if (null != ex.InnerException)
                    {
                        LogSys.ErrorModule("Exception", $"InnerException: {ex.InnerException.Message}. Stack {ex.InnerException.StackTrace}");
                    }
                    continue;
                }

                //LogSys.Debug("Env.Start2");

                TimeSpan span = DateTime.Now - startTime;
                if (span.TotalMilliseconds > 100.0f)
                {
                    LogSys.Warn($"[ServerUpdate]delta = {span.TotalMilliseconds}");
                }

                if (span.TotalMilliseconds < 10.0f)
                {
                    Thread.Sleep(1);
                }

                //LogSys.Debug("Env.Start3");
                //LogSys.Debug("==========================================================");
            }

            LogSys.Warn("Env.Start, Destroy");
            Destroy();

            return true;
        }

        public static bool Init()
        {
            Console.WriteLine($"[RunOnCurrentThread] Init, =================================================, threadid={System.Threading.Thread.CurrentThread.ManagedThreadId}");

            SynchronizationContext Current = SynchronizationContext.Current;

            // 设置同步上下文环境
            m_SynchronizationContext = new SingleThreadSynchronizationContext();
            SynchronizationContext.SetSynchronizationContext(m_SynchronizationContext);

            if (!otherConfig.Init()) return false;
            if (!weixin.Init()) return false;
            //if (!qqMini.Init())return false;
            LogSys.Init(Config.logCfg);
            //任务定时器初始化
            JobTimer.Init();

            Log.LogSys.Debug($"心跳包检测间隔时长{Config.HeartSec}秒");

            if (!g.Init()) return false;
            if (!ModuleManager.Init()) return false;

            playerMgr.Init();

            // 服务器开启时间
            ServerOpenTime = DateTime.Now;

            //double seed = 5;
            //LogSys.Debug("ishint" + RandomHelp.IsHit(seed));
            //for (int i = 0; i < 100; i++)
            //{
            //    LogSys.Info("random:" + RandomHelp.RandomSeed(ref seed));
            //}

            //JobTimerHelp.TestRankReward();//测试奖励

            return true;
        }

        public static void Update()
        {
            // 执行await后的回调函数
            SingleThreadSynchronizationContext context = SynchronizationContext.Current as SingleThreadSynchronizationContext;
            context.RunOnCurrentThread();

            ModuleManager.Tick();
            Timer.Update();
        }

        public static void Destroy()
        {
            ModuleManager.Destroy();
            JobTimer.Destroy();
        }
    }

    class g
    {
        //代币管理器
        // public static TokenManager TokenMgr = new TokenManager();
        //公告管理器
        // public static NoticeManager NoticeMgr = new NoticeManager();
        //滚动公告管理器
        // public static ScrollNoticeManager ScrollNoticeMgr = new ScrollNoticeManager();
        //邮件管理器
        // public static MailManager MailMgr = new MailManager();
        //平行世界管理器
        // public static WorldManager WorldMgr = new WorldManager();
        //银行管理器
        // public static BankManager BankMgr = new BankManager();
        //交易所管理器
        // public static BusinessManager BusinessMgr = new BusinessManager();
        //家园建造管理器
        // public static HomeBuildManager HomeBuildMgr = new HomeBuildManager();
        //房产系统管理器
        // public static HouseSysManager HouseSysMgr = new HouseSysManager();
        // public static AmaManager AmaManager = new AmaManager();
        /// <summary>
        /// 朋友圈
        /// </summary>
        /// <returns></returns>
        // public static FriendCircleManager FriendCircleMgr = new FriendCircleManager();

        /// <summary>
        /// 初始化
        /// </summary>
        /// <returns></returns>
        public static bool Init()
        {
            // TokenMgr.Init();
            // NoticeMgr.Init();
            // MailMgr.Init();
            // if (!ScrollNoticeMgr.Init()) return false;
            // if (!WorldMgr.Init()) return false;
            // if (!BankMgr.Init()) return false;
            // if (!BusinessMgr.Init()) return false;
            // if (!HomeBuildMgr.Init()) return false;
            // if (!HouseSysMgr.Init()) return false;
            // if (!AmaManager.Init()) return false;
            // if(!FriendCircleMgr.Init()) return false;
            return true;
        }
    }
}
